<?php

namespace Litecms\Lead\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Lead\Forms\Lead as LeadForm;
use Litecms\Lead\Http\Requests\LeadRequest;
use Litecms\Lead\Interfaces\LeadRepositoryInterface;
use Litecms\Lead\Repositories\Eloquent\Filters\LeadResourceFilter;
use Litecms\Lead\Repositories\Eloquent\Presenters\LeadListPresenter;

/**
 * Resource controller class for lead.
 */
class LeadResourceController extends BaseController
{

    /**
     * Initialize lead resource controller.
     *
     *
     * @return null
     */
    public function __construct(LeadRepositoryInterface $lead)
    {
        parent::__construct();
        $this->form = LeadForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.lead.modules'), 'lead', guard_url('lead'));
        $this->repository = $lead;
    }

    /**
     * Display a list of lead.
     *
     * @return Response
     */
    public function index(LeadRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(LeadResourceFilter::class)
            ->setPresenter(LeadListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('lead::lead.names'))
            ->view('lead::lead.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display lead.
     *
     * @param Request $request
     * @param Model   $lead
     *
     * @return Response
     */
    public function show(LeadRequest $request, LeadRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('lead::lead.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('lead::lead.show')
            ->output();
    }

    /**
     * Show the form for creating a new lead.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(LeadRequest $request, LeadRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('lead::lead.name'))
            ->view('lead::lead.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new lead.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeadRequest $request, LeadRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('lead::lead.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('lead/lead/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/lead/lead'))
                ->redirect();
        }

    }

    /**
     * Show lead for editing.
     *
     * @param Request $request
     * @param Model   $lead
     *
     * @return Response
     */
    public function edit(LeadRequest $request, LeadRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('lead::lead.name'))
            ->view('lead::lead.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the lead.
     *
     * @param Request $request
     * @param Model   $lead
     *
     * @return Response
     */
    public function update(LeadRequest $request, LeadRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('lead::lead.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('lead/lead/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('lead/lead/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the lead.
     *
     * @param Model   $lead
     *
     * @return Response
     */
    public function destroy(LeadRequest $request, LeadRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('lead::lead.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('lead/lead/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('lead/lead/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
